@@ -11,12 +11,10 @@ console.setLevel(logging.DEBUG)
 logger.addHandler(console)
 logger.setLevel(logging.ERROR)
 
-
-from .blocks.mdf_v2 import MDF2
-from .blocks.mdf_v3 import MDF3
-from .blocks.mdf_v4 import MDF4
-from .mdf import MDF, SUPPORTED_VERSIONS
+from .blocks.options import get_global_option, set_global_option
+from .blocks.source_utils import Source
 from .gui import plot
+from .mdf import MDF, SUPPORTED_VERSIONS
 from .signal import Signal
 from .version import __version__
 
@@ -27,15 +25,14 @@ try:
 except ImportError:
     __cextension__ = False
 
-
 __all__ = [
     "__cextension__",
     "__version__",
+    "get_global_option",
+    "set_global_option",
     "MDF",
-    "MDF2",
-    "MDF3",
-    "MDF4",
-    "Signal",
-    "SUPPORTED_VERSIONS",
     "plot",
+    "Signal",
+    "Source",
+    "SUPPORTED_VERSIONS",
 ]
