@@ -1,10 +1,79 @@
+from __future__ import annotations
+
+from traceback import format_exc
+from typing import Any
+
+from canmatrix import Frame, Signal
 import numpy as np
+from numpy.typing import NDArray
+from typing_extensions import TypedDict
 
 from .conversion_utils import from_dict
 from .utils import as_non_byte_sized_signed_int, MdfException
 
+MAX_VALID_J1939 = {
+    2: 1,
+    4: 0xA,
+    8: 0xFA,
+    10: 0x3FA,
+    12: 0xFAF,
+    16: 0xFAFF,
+    20: 0xFAFFF,
+    24: 0xFAFFFF,
+    28: 0xFAFFFFF,
+    32: 0xFAFFFFFF,
+    64: 0xFFFFFFFFFFFFFFFF,
+}
 
-def extract_signal(signal, payload, raw=False, ignore_value2text_conversion=True):
+
+def defined_j1939_bit_count(signal):
+    size = signal.size
+    for defined_size in (2, 4, 8, 10, 12, 16, 20, 24, 28, 32, 64):
+        if size <= defined_size:
+            return defined_size
+    return size
+
+
+def apply_conversion(
+    vals: NDArray[Any], signal: Signal, ignore_value2text_conversion: bool
+) -> NDArray[Any]:
+    a, b = float(signal.factor), float(signal.offset)
+
+    if signal.values:
+        if ignore_value2text_conversion:
+            if (a, b) != (1, 0):
+                vals = vals * a
+                if b:
+                    vals += b
+        else:
+
+            conv = {}
+            for i, (val, text) in enumerate(signal.values.items()):
+                conv[f"upper_{i}"] = val
+                conv[f"lower_{i}"] = val
+                conv[f"text_{i}"] = text
+
+            conv["default"] = from_dict({"a": a, "b": b})
+
+            conv = from_dict(conv)
+            vals = conv.convert(vals)
+
+    else:
+
+        if (a, b) != (1, 0):
+            vals = vals * a
+            if b:
+                vals += b
+
+    return vals
+
+
+def extract_signal(
+    signal: Signal,
+    payload: NDArray[Any],
+    raw: bool = False,
+    ignore_value2text_conversion: bool = True,
+) -> NDArray[Any]:
     vals = payload
 
     big_endian = False if signal.is_little_endian else True
@@ -106,7 +175,7 @@ def extract_signal(signal, payload, raw=False, ignore_value2text_conversion=True
 
             if std_size <= 8 and not is_float:
                 vals = vals >> (extra_bytes * 8 + bit_offset)
-                vals &= (2 ** bit_count) - 1
+                vals &= (2**bit_count) - 1
 
         else:
             vals = np.column_stack(
@@ -130,7 +199,7 @@ def extract_signal(signal, payload, raw=False, ignore_value2text_conversion=True
 
             if std_size <= 8 and not is_float:
                 vals = vals >> bit_offset
-                vals &= (2 ** bit_count) - 1
+                vals &= (2**bit_count) - 1
 
     else:
         if big_endian:
@@ -151,7 +220,7 @@ def extract_signal(signal, payload, raw=False, ignore_value2text_conversion=True
 
             if std_size <= 8 and not is_float:
                 vals = vals >> bit_offset
-                vals &= (2 ** bit_count) - 1
+                vals &= (2**bit_count) - 1
         else:
             if std_size > 8:
                 fmt = f"({std_size},)u1"
@@ -170,58 +239,58 @@ def extract_signal(signal, payload, raw=False, ignore_value2text_conversion=True
 
             if std_size <= 8 and not is_float:
                 vals = vals >> bit_offset
-                vals &= (2 ** bit_count) - 1
+                vals &= (2**bit_count) - 1
 
     if signed and not is_float:
         vals = as_non_byte_sized_signed_int(vals, bit_count)
 
     if not raw:
-        a, b = float(signal.factor), float(signal.offset)
-
-        if signal.values and not ignore_value2text_conversion:
-
-            conv = {}
-            for i, (val, text) in enumerate(signal.values.items()):
-                conv[f"upper_{i}"] = val
-                conv[f"lower_{i}"] = val
-                conv[f"text_{i}"] = text
-
-            conv["default"] = from_dict({"a": a, "b": b})
-
-            conv = from_dict(conv)
-            vals = conv.convert(vals)
-
-        else:
-
-            if (a, b) != (1, 0):
-                vals = vals * a
-                if b:
-                    vals += b
+        vals = apply_conversion(vals, signal, ignore_value2text_conversion)
 
     return vals
 
 
-def extract_can_signal(signal, payload, raw=False, ignore_value2text_conversion=True):
+def extract_can_signal(
+    signal: Signal,
+    payload: NDArray[Any],
+    raw: bool = False,
+    ignore_value2text_conversion: bool = True,
+) -> NDArray[Any]:
     return extract_signal(signal, payload, raw, ignore_value2text_conversion)
 
 
-def extract_lin_signal(signal, payload, raw=False, ignore_value2text_conversion=True):
+def extract_lin_signal(
+    signal: Signal,
+    payload: NDArray[Any],
+    raw: bool = False,
+    ignore_value2text_conversion: bool = True,
+) -> NDArray[Any]:
     return extract_signal(signal, payload, raw, ignore_value2text_conversion)
 
 
+class ExtractedSignal(TypedDict):
+    name: str
+    comment: str
+    unit: str
+    samples: NDArray[Any]
+    t: NDArray[Any]
+    invalidation_bits: NDArray[Any]
+
+
 def extract_mux(
-    payload,
-    message,
-    message_id,
-    bus,
-    t,
-    muxer=None,
-    muxer_values=None,
-    original_message_id=None,
-    raw=False,
-    include_message_name=False,
-    ignore_value2text_conversion=True,
-):
+    payload: NDArray[Any],
+    message: Frame,
+    message_id: int,
+    bus: int,
+    t: NDArray[Any],
+    muxer: str | None = None,
+    muxer_values: NDArray[Any] | None = None,
+    original_message_id: int | None = None,
+    raw: bool = False,
+    include_message_name: bool = False,
+    ignore_value2text_conversion: bool = True,
+    is_j1939: bool = False,
+) -> dict[tuple[Any, ...], dict[str, ExtractedSignal]]:
     """extract multiplexed CAN signals from the raw payload
 
     Parameters
@@ -301,12 +370,15 @@ def extract_mux(
             payload_ = payload
 
         for sig in pair_signals:
-            samples = extract_signal(sig, payload_, raw, ignore_value2text_conversion)
+            samples = extract_signal(
+                sig,
+                payload_,
+                ignore_value2text_conversion=ignore_value2text_conversion,
+                raw=True,
+            )
             if len(samples) == 0 and len(t_):
                 continue
 
-            max_val = np.full(len(samples), float(sig.calc_max()))
-
             if include_message_name:
                 sig_name = f"{message.name}.{sig.name}"
             else:
@@ -317,18 +389,22 @@ def extract_mux(
                     "name": sig_name,
                     "comment": sig.comment or "",
                     "unit": sig.unit or "",
-                    "samples": samples,
+                    "samples": samples
+                    if raw
+                    else apply_conversion(samples, sig, ignore_value2text_conversion),
                     "t": t_,
-                    "invalidation_bits": (
-                        np.isclose(samples, max_val)
-                        if len(samples.shape) == 1
-                        else np.zeros(len(samples), dtype=bool)
-                    ),
+                    "invalidation_bits": None,
                 }
+
+                if is_j1939:
+                    signals[sig_name]["invalidation_bits"] = (
+                        samples > MAX_VALID_J1939[defined_j1939_bit_count(sig)]
+                    )
+
             except:
+                print(format_exc())
                 print(message, sig)
                 print(samples, set(samples), samples.dtype, samples.shape)
-                print(max_val, max_val.dtype, max_val.shape)
                 raise
 
             if sig.multiplex == "Multiplexor":
@@ -343,6 +419,8 @@ def extract_mux(
                         muxer_values=samples,
                         original_message_id=original_message_id,
                         ignore_value2text_conversion=ignore_value2text_conversion,
+                        raw=raw,
+                        is_j1939=is_j1939,
                     )
                 )
 
