@@ -3,12 +3,15 @@
 asammdf utility functions for source information
 """
 
+from __future__ import annotations
+
+from functools import lru_cache
+
 from . import v2_v3_blocks as v3b
 from . import v2_v3_constants as v3c
 from . import v4_blocks as v4b
 from . import v4_constants as v4c
-
-__all__ = ["Source"]
+from ..types import SourceType
 
 
 class Source:
@@ -32,7 +35,9 @@ class Source:
     BUS_TYPE_ETHERNET = v4c.BUS_TYPE_ETHERNET
     BUS_TYPE_USB = v4c.BUS_TYPE_USB
 
-    def __init__(self, name, path, comment, source_type, bus_type):
+    def __init__(
+        self, name: str, path: str, comment: str, source_type: int, bus_type: int
+    ) -> None:
         """Commons reprezentation for source information
 
         Attributes
@@ -58,10 +63,11 @@ class Source:
         )
 
     @classmethod
-    def from_source(cls, source):
+    @lru_cache(128)
+    def from_source(cls, source: SourceType) -> Source:
         if isinstance(source, v3b.ChannelExtension):
             if source.type == v3c.SOURCE_ECU:
-                source = cls(
+                return cls(
                     source.name,
                     source.path,
                     source.comment,
@@ -69,7 +75,7 @@ class Source:
                     cls.BUS_TYPE_NONE,  # bus type none
                 )
             else:
-                source = cls(
+                return cls(
                     source.name,
                     source.path,
                     source.comment,
@@ -93,3 +99,8 @@ class Source:
                 source.source_type,
                 source.bus_type,
             )
+
+    def get_details(self) -> str:
+        source_type = v4c.SOURCE_TYPE_TO_STRING[self.source_type]
+        bus_type = v4c.BUS_TYPE_TO_STRING[self.bus_type]
+        return f"     {source_type} source on bus {bus_type}: name=[{self.name}] path=[{self.path}]"
