@@ -2,6 +2,7 @@
 """ MDF v4 constants """
 import re
 import struct
+import sys
 
 MAX_UINT64 = (1 << 64) - 1
 
@@ -23,6 +24,26 @@ DATA_TYPE_CANOPEN_TIME = 14
 DATA_TYPE_COMPLEX_INTEL = 15
 DATA_TYPE_COMPLEX_MOTOROLA = 16
 
+VALID_DATA_TYPES = {
+    DATA_TYPE_UNSIGNED_INTEL,
+    DATA_TYPE_UNSIGNED_MOTOROLA,
+    DATA_TYPE_SIGNED_INTEL,
+    DATA_TYPE_SIGNED_MOTOROLA,
+    DATA_TYPE_REAL_INTEL,
+    DATA_TYPE_REAL_MOTOROLA,
+    DATA_TYPE_STRING_LATIN_1,
+    DATA_TYPE_STRING_UTF_8,
+    DATA_TYPE_STRING_UTF_16_LE,
+    DATA_TYPE_STRING_UTF_16_BE,
+    DATA_TYPE_BYTEARRAY,
+    DATA_TYPE_MIME_SAMPLE,
+    DATA_TYPE_MIME_STREAM,
+    DATA_TYPE_CANOPEN_DATE,
+    DATA_TYPE_CANOPEN_TIME,
+    DATA_TYPE_COMPLEX_INTEL,
+    DATA_TYPE_COMPLEX_MOTOROLA,
+}
+
 DATA_TYPE_TO_STRING = dict(
     enumerate(
         [
@@ -262,6 +283,7 @@ FH_BLOCK_SIZE = 56
 DG_BLOCK_SIZE = 64
 HD_BLOCK_SIZE = 104
 CN_BLOCK_SIZE = 160
+CN_SINGLE_ATTACHMENT_BLOCK_SIZE = 168
 CG_BLOCK_SIZE = 104
 CG_RM_BLOCK_SIZE = 112
 COMMON_SIZE = 24
@@ -271,6 +293,7 @@ CC_ALG_BLOCK_SIZE = 88
 CC_LIN_BLOCK_SIZE = 96
 AT_COMMON_SIZE = 96
 DZ_COMMON_SIZE = 48
+DZ_COMMON_INFO_SIZE = 22
 CC_COMMON_BLOCK_SIZE = 80
 HL_BLOCK_SIZE = 40
 IDENTIFICATION_BLOCK_SIZE = 64
@@ -395,6 +418,11 @@ DZ_BLOCK_LZ = 3
 FMT_CHANNEL = "<4sI2Q{}Q4B4I2BH6d"
 FMT_CHANNEL_PARAMS = "<4B4I2BH6d"
 
+FMT_CHANNEL_FILTER = "<4s12x4Q32xQ"
+CHANNEL_FILTER_u = struct.Struct(FMT_CHANNEL_FILTER).unpack
+CHANNEL_FILTER_uf = struct.Struct(FMT_CHANNEL_FILTER).unpack_from
+CHANNEL_FILTER_SIZE = 88
+
 FMT_SIMPLE_CHANNEL = "<4sI10Q4B4I2BH6d"
 KEYS_SIMPLE_CHANNEL = (
     "id",
@@ -431,6 +459,15 @@ FMT_SIMPLE_CHANNEL_PARAMS = "<8Q4B4I2BH6d"
 SIMPLE_CHANNEL_PARAMS_u = struct.Struct(FMT_SIMPLE_CHANNEL_PARAMS).unpack
 SIMPLE_CHANNEL_PARAMS_uf = struct.Struct(FMT_SIMPLE_CHANNEL_PARAMS).unpack_from
 SIMPLE_CHANNEL_PACK = struct.Struct(FMT_SIMPLE_CHANNEL).pack
+FMT_SINGLE_ATTACHMENT_CHANNEL = "<4sI11Q4B4I2BH6d"
+SINGLE_ATTACHMENT_CHANNEL_PACK = struct.Struct(FMT_SINGLE_ATTACHMENT_CHANNEL).pack
+FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS = "<9Q4B4I2BH6d"
+SINGLE_ATTACHMENT_CHANNEL_PARAMS_uf = struct.Struct(
+    FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS
+).unpack_from
+SINGLE_ATTACHMENT_CHANNEL_PARAMS_u = struct.Struct(
+    FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS
+).unpack
 
 FMT_TEXT_BLOCK = "<4sIQQ{}s"
 KEYS_TEXT_BLOCK = ("id", "reserved0", "block_len", "links_nr", "text")
@@ -721,7 +758,8 @@ KEYS_DZ_BLOCK = (
 )
 DZ_COMMON_p = struct.Struct(FMT_DZ_COMMON).pack
 
-FMT_DZ_INFO_COMMON = "<24x2sB1xI2Q"
+FMT_DZ_INFO_COMMON = "<B1xI2Q"
+DZ_INFO_COMMON_OFFSET = 26
 DZ_COMMON_INFO_u = struct.Struct(FMT_DZ_INFO_COMMON).unpack
 DZ_COMMON_INFO_uf = struct.Struct(FMT_DZ_INFO_COMMON).unpack_from
 
@@ -764,7 +802,7 @@ LIN_DATA_FRAME_PATTERN = re.compile(r"(?i:LIN_Frame)_(?P<id>\d+)")
 CN_COMMENT_TEMPLATE = """<CNcomment>
 <TX>{}</TX>
 <names>
-    <display>{}</display>
+{}
 </names>
 </CNcomment>"""
 
@@ -789,3 +827,39 @@ CANOPEN_DATE_FIELDS = (
     "summer_time",
     "day_of_week",
 )
+
+CAN_ERROR_TYPES = {
+    0: "Other/unknown error ",
+    1: "Bit Error",
+    2: "Form Error",
+    3: "Bit Stuffing Error",
+    4: "CRC Error",
+    5: "Acknowledgment Error",
+}
+
+for string in (
+    "CAN Frame",
+    "Error Frame",
+    "Remote Frame",
+    "Unknown",
+    "LIN Frame",
+    "Checksum Error Frame",
+    "Receive Error Frame",
+    "Sync Error Frame",
+    "Transmission Error Frame",
+    "A",
+    "B",
+    "AB",
+):
+    sys.intern(string)
+
+for string in CAN_ERROR_TYPES.values():
+    sys.intern(string)
+
+for chn in range(20):
+    sys.intern(f"CAN {chn}")
+    sys.intern(f"LIN {chn}")
+    sys.intern(f"FLX {chn}")
+
+for chk in range(256):
+    sys.intern(f"Checksum 0x{chk:02X}")
