@@ -1,29 +1,42 @@
 # -*- coding: utf-8 -*-
+from datetime import datetime, timedelta, timezone
 import os
 from pathlib import Path
 
 from natsort import natsorted
 import psutil
-from PyQt5 import QtCore, QtGui, QtWidgets
+from PySide6 import QtCore, QtGui, QtWidgets
 
-from ...blocks.utils import extract_cncomment_xml
+from ...blocks.utils import extract_xml_comment
+from ...blocks.v2_v3_blocks import HeaderBlock as HeaderBlockV3
+from ...blocks.v4_blocks import HeaderBlock as HeaderBlockV4
 from ...mdf import MDF, SUPPORTED_VERSIONS
 from ..dialogs.advanced_search import AdvancedSearch
-from ..ui import resource_rc as resource_rc
+from ..ui import resource_rc
 from ..ui.batch_widget import Ui_batch_widget
 from ..utils import (
-    add_children,
     HelperChannel,
+    load_dsp,
+    load_lab,
     run_thread_with_progress,
     setup_progress,
     TERMINATED,
 )
+from .database_item import DatabaseItem
 from .list import MinimalListWidget
+from .tree import add_children
 from .tree_item import TreeItem
 
 
 class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
-    def __init__(self, ignore_value2text_conversions=False, *args, **kwargs):
+    def __init__(
+        self,
+        ignore_value2text_conversions=False,
+        integer_interpolation=2,
+        float_interpolation=1,
+        *args,
+        **kwargs,
+    ):
 
         super().__init__(*args, **kwargs)
         self.setupUi(self)
@@ -31,10 +44,19 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         self._settings = QtCore.QSettings()
 
         self.ignore_value2text_conversions = ignore_value2text_conversions
+        self.integer_interpolation = integer_interpolation
+        self.float_interpolation = float_interpolation
+
+        self.files_list.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
 
         self.progress = None
-        self.files_list = MinimalListWidget()
-        self.splitter.addWidget(self.files_list)
+        self.show()
+
+        sizes = sum(self.splitter.sizes())
+        if sizes >= 700:
+            self.splitter.setSizes([700, sizes - 700])
+        self.splitter.setStretchFactor(0, 0)
+        self.splitter.setStretchFactor(1, 1)
 
         self.raster_type_channel.toggled.connect(self.set_raster_type)
 
@@ -45,6 +67,7 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
             self.mdf_version,
         ):
             widget.insertItems(0, SUPPORTED_VERSIONS)
+            widget.setCurrentText("4.10")
 
         for widget in (
             self.concatenate_split_size,
@@ -61,6 +84,7 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         ):
 
             widget.insertItems(0, ("no compression", "deflate", "transposed deflate"))
+            widget.setCurrentText("transposed deflate")
 
         self.concatenate_btn.clicked.connect(self.concatenate)
         self.scramble_btn.clicked.connect(self.scramble)
@@ -72,6 +96,8 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         self.apply_btn.clicked.connect(self.apply_processing)
         self.modify_output_folder_btn.clicked.connect(self.change_modify_output_folder)
         self.output_format.currentTextChanged.connect(self.output_format_changed)
+        self.sort_alphabetically_btn.clicked.connect(self.sort_alphabetically)
+        self.sort_by_start_time_btn.clicked.connect(self.sort_by_start_time)
 
         self.filter_view.setCurrentIndex(-1)
         self.filter_view.currentIndexChanged.connect(self._update_channel_tree)
@@ -83,7 +109,21 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         self.load_can_database_btn.clicked.connect(self.load_can_database)
         self.load_lin_database_btn.clicked.connect(self.load_lin_database)
 
+        self.load_filter_list_btn.clicked.connect(self.load_filter_list)
+        self.save_filter_list_btn.clicked.connect(self.save_filter_list)
+
         self.empty_channels_bus.insertItems(0, ("skip", "zeros"))
+        self.empty_channels.insertItems(0, ("skip", "zeros"))
+        self.empty_channels_bus.insertItems(0, ("skip", "zeros"))
+        self.empty_channels_mat.insertItems(0, ("skip", "zeros"))
+        self.empty_channels_csv.insertItems(0, ("skip", "zeros"))
+        try:
+            import scipy
+
+            self.mat_format.insertItems(0, ("4", "5", "7.3"))
+        except:
+            self.mat_format.insertItems(0, ("7.3",))
+        self.oned_as.insertItems(0, ("row", "column"))
 
         self.aspects.setCurrentIndex(0)
         self.setAcceptDrops(True)
@@ -174,27 +214,33 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
 
         database_files = {}
 
-        count = self.can_database_list.count()
-        if count:
+        count1 = self.can_database_list.count()
+        if count1:
             database_files["CAN"] = []
-            for i in range(count):
+            for i in range(count1):
                 item = self.can_database_list.item(i)
-                database_files["CAN"].append(item.text())
+                widget = self.can_database_list.itemWidget(item)
+                database_files["CAN"].append(
+                    (widget.database.text(), widget.bus.currentIndex())
+                )
 
-        count = self.lin_database_list.count()
-        if count:
+        count2 = self.lin_database_list.count()
+        if count2:
             database_files["LIN"] = []
-            for i in range(count):
+            for i in range(count2):
                 item = self.lin_database_list.item(i)
-                database_files["LIN"].append(item.text())
+                widget = self.lin_database_list.itemWidget(item)
+                database_files["LIN"].append(
+                    (widget.database.text(), widget.bus.currentIndex())
+                )
 
         compression = self.extract_bus_compression.currentIndex()
-        ignore_invalid_signals = (
-            self.ignore_invalid_signals_mdf.checkState() == QtCore.Qt.Checked
-        )
 
         count = self.files_list.count()
 
+        if not count or not (count1 + count2):
+            return
+
         delta = 100 / count
 
         progress = setup_progress(
@@ -239,7 +285,9 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
             kwargs = {
                 "database_files": database_files,
                 "version": version,
-                "ignore_invalid_signals": ignore_invalid_signals,
+                "prefix": self.prefix.text().strip(),
+                "consolidated_j1939": self.consolidated_j1939.checkState()
+                == QtCore.Qt.Checked,
             }
 
             mdf_ = run_thread_with_progress(
@@ -323,31 +371,46 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
 
         database_files = {}
 
-        count = self.can_database_list.count()
-        if count:
+        count1 = self.can_database_list.count()
+        if count1:
             database_files["CAN"] = []
-            for i in range(count):
+            for i in range(count1):
                 item = self.can_database_list.item(i)
-                database_files["CAN"].append(item.text())
+                widget = self.can_database_list.itemWidget(item)
+                database_files["CAN"].append(
+                    (widget.database.text(), widget.bus.currentIndex())
+                )
 
-        count = self.lin_database_list.count()
-        if count:
+        count2 = self.lin_database_list.count()
+        if count2:
             database_files["LIN"] = []
-            for i in range(count):
+            for i in range(count2):
                 item = self.lin_database_list.item(i)
-                database_files["LIN"].append(item.text())
+                widget = self.lin_database_list.itemWidget(item)
+                database_files["LIN"].append(
+                    (widget.database.text(), widget.bus.currentIndex())
+                )
 
-        ignore_invalid_signals = (
-            self.ignore_invalid_signals_csv.checkState() == QtCore.Qt.Checked
-        )
         single_time_base = self.single_time_base_bus.checkState() == QtCore.Qt.Checked
         time_from_zero = self.time_from_zero_bus.checkState() == QtCore.Qt.Checked
         empty_channels = self.empty_channels_bus.currentText()
         raster = self.export_raster_bus.value() or None
         time_as_date = self.bus_time_as_date.checkState() == QtCore.Qt.Checked
+        delimiter = self.delimiter_bus.text() or ","
+        doublequote = self.doublequote_bus.checkState() == QtCore.Qt.Checked
+        escapechar = self.escapechar_bus.text() or None
+        lineterminator = (
+            self.lineterminator_bus.text().replace("\\r", "\r").replace("\\n", "\n")
+        )
+        quotechar = self.quotechar_bus.text() or '"'
+        quoting = self.quoting_bus.currentText()
+        add_units = self.add_units_bus.checkState() == QtCore.Qt.Checked
 
         count = self.files_list.count()
 
+        if not count or not (count1 + count2):
+            return
+
         delta = 100 / count
 
         progress = setup_progress(
@@ -392,7 +455,9 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
             kwargs = {
                 "database_files": database_files,
                 "version": version,
-                "ignore_invalid_signals": ignore_invalid_signals,
+                "prefix": self.prefix.text().strip(),
+                "consolidated_j1939": self.consolidated_j1939.checkState()
+                == QtCore.Qt.Checked,
             }
 
             mdf_ = run_thread_with_progress(
@@ -450,6 +515,11 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                 f'Saving extracted Bus logging file {i+1} to "{file_name}"'
             )
 
+            mdf_.configure(
+                integer_interpolation=self.integer_interpolation,
+                float_interpolation=self.float_interpolation,
+            )
+
             target = mdf_.export
             kwargs = {
                 "fmt": "csv",
@@ -460,6 +530,13 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                 "raster": raster or None,
                 "time_as_date": time_as_date,
                 "ignore_value2text_conversions": self.ignore_value2text_conversions,
+                "delimiter": delimiter,
+                "doublequote": doublequote,
+                "escapechar": escapechar,
+                "lineterminator": lineterminator,
+                "quotechar": quotechar,
+                "quoting": quoting,
+                "add_units": add_units,
             }
 
             run_thread_with_progress(
@@ -479,26 +556,38 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
     def load_can_database(self, event):
         file_names, _ = QtWidgets.QFileDialog.getOpenFileNames(
             self,
-            "Select Bus database file",
+            "Select CAN database file",
             "",
-            "ARXML or DBC (*.dbc *.axml)",
-            "ARXML or DBC (*.dbc *.axml)",
+            "ARXML or DBC (*.dbc *.arxml)",
+            "ARXML or DBC (*.dbc *.arxml)",
         )
 
         if file_names:
-            self.can_database_list.addItems(file_names)
+            for database in file_names:
+                item = QtWidgets.QListWidgetItem()
+                widget = DatabaseItem(database, bus_type="CAN")
+
+                self.can_database_list.addItem(item)
+                self.can_database_list.setItemWidget(item, widget)
+                item.setSizeHint(widget.sizeHint())
 
     def load_lin_database(self, event):
         file_names, _ = QtWidgets.QFileDialog.getOpenFileNames(
             self,
             "Select LIN database file",
             "",
-            "ARXML or DBC (*.dbc *.arxml)",
-            "ARXML or DBC (*.dbc *.arxml)",
+            "ARXML or DBC database (*.dbc *.arxml);;LDF database (*.ldf);;All supported formats (*.dbc *.arxml *ldf)",
+            "All supported formats (*.dbc *.arxml *ldf)",
         )
 
         if file_names:
-            self.lin_database_list.addItems(file_names)
+            for database in file_names:
+                item = QtWidgets.QListWidgetItem()
+                widget = DatabaseItem(database, bus_type="LIN")
+
+                self.lin_database_list.addItem(item)
+                self.lin_database_list.setItemWidget(item, widget)
+                item.setSizeHint(widget.sizeHint())
 
     def concatenate(self, event):
         count = self.files_list.count()
@@ -792,21 +881,20 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         if not self.files_list.count():
             return
 
-        mdf = MDF(self.files_list.item(0).text())
-        channels = self.mdf.channels_db
-        mdf.close()
-        dlg = AdvancedSearch(
-            channels, show_add_window=False, show_pattern=False, parent=self
-        )
-        dlg.setModal(True)
-        dlg.exec_()
-        result = dlg.result
-        if result:
-            dg_cntr, ch_cntr = next(iter(result))
-
-            name = self.mdf.groups[dg_cntr].channels[ch_cntr].name
-
-            self.raster_channel.setCurrentText(name)
+        with MDF(self.files_list.item(0).text()) as mdf:
+            dlg = AdvancedSearch(
+                mdf,
+                show_add_window=False,
+                show_pattern=False,
+                parent=self,
+                return_names=True,
+            )
+            dlg.setModal(True)
+            dlg.exec_()
+            result = dlg.result
+            if result:
+                name = list(result)[0]
+                self.raster_channel.setCurrentText(name)
 
     def filter_changed(self, item, column):
         name = item.text(0)
@@ -825,83 +913,83 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         if not self.files_list.count():
             return
 
-        mdf = MDF(self.files_list.item(0).text())
-        channels = self.mdf.channels_db
-        mdf.close()
+        with MDF(self.files_list.item(0).text()) as mdf:
 
-        widget = self.filter_tree
-        view = self.filter_view
+            widget = self.filter_tree
+            view = self.filter_view
 
-        dlg = AdvancedSearch(
-            channels, show_add_window=False, show_pattern=False, parent=self
-        )
-        dlg.setModal(True)
-        dlg.exec_()
-        result = dlg.result
+            dlg = AdvancedSearch(
+                mdf,
+                show_add_window=False,
+                show_pattern=False,
+                parent=self,
+                show_apply=True,
+                apply_text="Check signals",
+            )
+            dlg.setModal(True)
+            dlg.exec_()
+            result = dlg.result
 
-        if result:
-            names = set()
-            if view.currentText() == "Internal file structure":
-                iterator = QtWidgets.QTreeWidgetItemIterator(widget)
+            if result:
+                if view.currentText() == "Internal file structure":
+                    iterator = QtWidgets.QTreeWidgetItemIterator(widget)
 
-                dg_cntr = -1
-                ch_cntr = 0
+                    dg_cntr = -1
+                    ch_cntr = 0
+
+                    while iterator.value():
+                        item = iterator.value()
+                        if item.parent() is None:
+                            iterator += 1
+                            dg_cntr += 1
+                            ch_cntr = 0
+                            continue
+
+                        if (dg_cntr, ch_cntr) in result:
+                            item.setCheckState(0, QtCore.Qt.Checked)
 
-                while iterator.value():
-                    item = iterator.value()
-                    if item.parent() is None:
                         iterator += 1
-                        dg_cntr += 1
-                        ch_cntr = 0
-                        continue
+                        ch_cntr += 1
+                elif view.currentText() == "Selected channels only":
+                    iterator = QtWidgets.QTreeWidgetItemIterator(widget)
 
-                    if (dg_cntr, ch_cntr) in result:
-                        item.setCheckState(0, QtCore.Qt.Checked)
-                        names.add(item.text(0))
+                    signals = set()
+                    while iterator.value():
+                        item = iterator.value()
 
-                    iterator += 1
-                    ch_cntr += 1
-            elif view.currentText() == "Selected channels only":
-                iterator = QtWidgets.QTreeWidgetItemIterator(self.filter_tree)
+                        if item.checkState(0) == QtCore.Qt.Checked:
+                            signals.add(item.entry)
 
-                signals = set()
-                while iterator.value():
-                    item = iterator.value()
+                        iterator += 1
 
-                    if item.checkState(0) == QtCore.Qt.Checked:
-                        signals.add(item.entry)
+                    signals = signals | set(result)
 
-                    iterator += 1
+                    widget.clear()
 
-                signals = signals | result
+                    items = []
+                    for entry in signals:
+                        gp_index, ch_index = entry
+                        ch = mdf.groups[gp_index].channels[ch_index]
+                        channel = TreeItem(entry, ch.name, origin_uuid=self.uuid)
+                        channel.setText(0, ch.name)
+                        channel.setCheckState(0, QtCore.Qt.Checked)
+                        items.append(channel)
 
-                widget.clear()
+                    if len(items) < 30000:
+                        items = natsorted(items, key=lambda x: x.name)
+                    else:
+                        items.sort(key=lambda x: x.name)
+                    widget.addTopLevelItems(items)
 
-                items = []
-                for entry in signals:
-                    gp_index, ch_index = entry
-                    ch = self.mdf.groups[gp_index].channels[ch_index]
-                    channel = TreeItem(entry, ch.name, mdf_uuid=self.uuid)
-                    channel.setText(0, ch.name)
-                    channel.setCheckState(0, QtCore.Qt.Checked)
-                    items.append(channel)
-
-                if len(items) < 30000:
-                    items = natsorted(items, key=lambda x: x.name)
                 else:
-                    items.sort(key=lambda x: x.name)
-                widget.addTopLevelItems(items)
+                    iterator = QtWidgets.QTreeWidgetItemIterator(widget)
+                    while iterator.value():
+                        item = iterator.value()
 
-            else:
-                iterator = QtWidgets.QTreeWidgetItemIterator(widget)
-                while iterator.value():
-                    item = iterator.value()
+                        if item.entry in result:
+                            item.setCheckState(0, QtCore.Qt.Checked)
 
-                    if item.entry in result:
-                        item.setCheckState(0, QtCore.Qt.Checked)
-                        names.add(item.text(0))
-
-                    iterator += 1
+                        iterator += 1
 
     def _update_channel_tree(self, index=None):
         if self.filter_view.currentIndex() == -1:
@@ -949,7 +1037,7 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                         for j, ch in enumerate(group.channels):
                             entry = i, j
 
-                            channel = TreeItem(entry, ch.name, mdf_uuid=uuid)
+                            channel = TreeItem(entry, ch.name, origin_uuid=uuid)
                             channel.setText(0, ch.name)
                             if entry in signals:
                                 channel.setCheckState(0, QtCore.Qt.Checked)
@@ -966,9 +1054,9 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                 elif widget.mode == "Internal file structure":
                     for i, group in enumerate(mdf.groups):
                         entry = i, 0xFFFFFFFFFFFFFFFF
-                        channel_group = TreeItem(entry, mdf_uuid=uuid)
+                        channel_group = TreeItem(entry, origin_uuid=uuid)
                         comment = group.channel_group.comment
-                        comment = extract_cncomment_xml(comment)
+                        comment = extract_xml_comment(comment)
 
                         if comment:
                             channel_group.setText(0, f"Channel group {i} ({comment})")
@@ -976,7 +1064,7 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                             channel_group.setText(0, f"Channel group {i}")
                         channel_group.setFlags(
                             channel_group.flags()
-                            | QtCore.Qt.ItemIsTristate
+                            | QtCore.Qt.ItemIsAutoTristate
                             | QtCore.Qt.ItemIsUserCheckable
                         )
 
@@ -993,14 +1081,14 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                             group.channel_dependencies,
                             signals,
                             entries=None,
-                            mdf_uuid=uuid,
+                            origin_uuid=uuid,
                         )
                 else:
                     items = []
                     for entry in signals:
                         gp_index, ch_index = entry
                         ch = mdf.groups[gp_index].channels[ch_index]
-                        channel = TreeItem(entry, ch.name, mdf_uuid=uuid)
+                        channel = TreeItem(entry, ch.name, origin_uuid=uuid)
                         channel.setText(0, ch.name)
                         channel.setCheckState(0, QtCore.Qt.Checked)
                         items.append(channel)
@@ -1059,6 +1147,32 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                 "raw": self.raw_mat.checkState() == QtCore.Qt.Checked,
             }
 
+        elif output_format == "CSV":
+
+            new = {
+                "single_time_base": self.single_time_base_csv.checkState()
+                == QtCore.Qt.Checked,
+                "time_from_zero": self.time_from_zero_csv.checkState()
+                == QtCore.Qt.Checked,
+                "time_as_date": self.time_as_date_csv.checkState() == QtCore.Qt.Checked,
+                "use_display_names": self.use_display_names_csv.checkState()
+                == QtCore.Qt.Checked,
+                "reduce_memory_usage": False,
+                "compression": False,
+                "empty_channels": self.empty_channels_csv.currentText(),
+                "raw": self.raw_csv.checkState() == QtCore.Qt.Checked,
+                "delimiter": self.delimiter.text() or ",",
+                "doublequote": self.doublequote.checkState() == QtCore.Qt.Checked,
+                "escapechar": self.escapechar.text() or None,
+                "lineterminator": self.lineterminator.text()
+                .replace("\\r", "\r")
+                .replace("\\n", "\n"),
+                "quotechar": self.quotechar.text() or '"',
+                "quoting": self.quoting.currentText(),
+                "mat_format": None,
+                "oned_as": None,
+            }
+
         else:
 
             new = {
@@ -1183,7 +1297,11 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
         split_size = opts.mdf_split_size if output_format == "MDF" else 0
 
         for i, (mdf_file, source_file) in enumerate(zip(files, source_files)):
-            mdf_file.configure(read_fragment_size=split_size)
+            mdf_file.configure(
+                read_fragment_size=split_size,
+                integer_interpolation=self.integer_interpolation,
+                float_interpolation=self.float_interpolation,
+            )
 
             mdf = None
             progress = None
@@ -1220,7 +1338,10 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                     mdf = result
 
                 mdf.configure(
-                    read_fragment_size=split_size, write_fragment_size=split_size
+                    read_fragment_size=split_size,
+                    write_fragment_size=split_size,
+                    integer_interpolation=self.integer_interpolation,
+                    float_interpolation=self.float_interpolation,
                 )
 
             if opts.needs_cut:
@@ -1273,7 +1394,10 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                         mdf = result
 
                 mdf.configure(
-                    read_fragment_size=split_size, write_fragment_size=split_size
+                    read_fragment_size=split_size,
+                    write_fragment_size=split_size,
+                    integer_interpolation=self.integer_interpolation,
+                    float_interpolation=self.float_interpolation,
                 )
 
             if opts.needs_resample:
@@ -1331,7 +1455,10 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                         mdf = result
 
                 mdf.configure(
-                    read_fragment_size=split_size, write_fragment_size=split_size
+                    read_fragment_size=split_size,
+                    write_fragment_size=split_size,
+                    integer_interpolation=self.integer_interpolation,
+                    float_interpolation=self.float_interpolation,
                 )
 
             if output_format == "MDF":
@@ -1381,7 +1508,10 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                     suffix = ".mdf"
 
                 mdf.configure(
-                    read_fragment_size=split_size, write_fragment_size=split_size
+                    read_fragment_size=split_size,
+                    write_fragment_size=split_size,
+                    integer_interpolation=self.integer_interpolation,
+                    float_interpolation=self.float_interpolation,
                 )
 
                 if output_folder is not None:
@@ -1458,6 +1588,16 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                 if not file_name.parent.exists():
                     os.makedirs(file_name.parent, exist_ok=True)
 
+                delimiter = self.delimiter.text() or ","
+                doublequote = self.doublequote.checkState() == QtCore.Qt.Checked
+                escapechar = self.escapechar.text() or None
+                lineterminator = (
+                    self.lineterminator.text().replace("\\r", "\r").replace("\\n", "\n")
+                )
+                quotechar = self.quotechar.text() or '"'
+                quoting = self.quoting.currentText()
+                add_units = self.add_units.checkState() == QtCore.Qt.Checked
+
                 target = mdf_file.export if mdf is None else mdf.export
                 kwargs = {
                     "fmt": opts.output_format.lower(),
@@ -1474,6 +1614,13 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
                     "time_as_date": opts.time_as_date,
                     "ignore_value2text_conversions": self.ignore_value2text_conversions,
                     "raw": opts.raw,
+                    "delimiter": delimiter,
+                    "doublequote": doublequote,
+                    "escapechar": escapechar,
+                    "lineterminator": lineterminator,
+                    "quotechar": quotechar,
+                    "quoting": quoting,
+                    "add_units": add_units,
                 }
 
                 result = run_thread_with_progress(
@@ -1505,6 +1652,8 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
             self.export_compression_mat.clear()
             self.export_compression_mat.addItems(["enabled", "disabled"])
             self.export_compression_mat.setCurrentIndex(0)
+        elif name == "CSV":
+            self.output_options.setCurrentIndex(3)
         else:
             self.output_options.setCurrentIndex(1)
             if name == "Parquet":
@@ -1520,3 +1669,220 @@ class BatchWidget(Ui_batch_widget, QtWidgets.QWidget):
             elif name:
                 self.export_compression.clear()
                 self.export_compression.setEnabled(False)
+
+    def sort_alphabetically(self, event=None):
+        count = self.files_list.count()
+
+        if not count:
+            return
+
+        source_files = natsorted(
+            [self.files_list.item(row).text() for row in range(count)]
+        )
+
+        self.files_list.clear()
+        self.files_list.addItems(source_files)
+
+        icon = QtGui.QIcon()
+        icon.addPixmap(QtGui.QPixmap(":/file.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
+        for row in range(count):
+            self.files_list.item(row).setIcon(icon)
+
+    def sort_by_start_time(self, event=None):
+        count = self.files_list.count()
+
+        if not count:
+            return
+
+        source_files = [self.files_list.item(row).text() for row in range(count)]
+
+        start_times = []
+
+        for file_name in source_files:
+            with open(file_name, "rb") as f:
+                f.seek(64)
+                blk_id = f.read(2)
+                block_type = HeaderBlockV4 if blk_id == b"##" else HeaderBlockV3
+                header = block_type(stream=f, address=64)
+                start_times.append((header.start_time, file_name))
+
+        try:
+            start_times = sorted(start_times)
+        except TypeError:
+            start_times = [
+                (st.replace(tzinfo=timezone.utc), name) for (st, name) in start_times
+            ]
+            start_times = sorted(start_times)
+
+        self.files_list.clear()
+        self.files_list.addItems([item[1] for item in start_times])
+
+        icon = QtGui.QIcon()
+        icon.addPixmap(QtGui.QPixmap(":/file.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
+        for row in range(count):
+            self.files_list.item(row).setIcon(icon)
+
+    def save_filter_list(self):
+        file_name, _ = QtWidgets.QFileDialog.getSaveFileName(
+            self,
+            "Select output filter list file",
+            self.default_folder,
+            "CANape Lab file (*.lab);;TXT files (*.txt);;All file types (*.lab *.txt)",
+            "CANape Lab file (*.lab)",
+        )
+
+        if file_name:
+            file_name = Path(file_name)
+
+            iterator = QtWidgets.QTreeWidgetItemIterator(self.filter_tree)
+
+            signals = []
+            if self.filter_view.currentText() == "Internal file structure":
+                while True:
+                    item = iterator.value()
+                    if item is None:
+                        break
+
+                    iterator += 1
+
+                    if item.parent() is None:
+                        continue
+
+                    if item.checkState(0) == QtCore.Qt.Checked:
+                        signals.append(item.text(0))
+            else:
+                while True:
+                    item = iterator.value()
+                    if item is None:
+                        break
+
+                    iterator += 1
+
+                    if item.checkState(0) == QtCore.Qt.Checked:
+                        signals.append(item.text(0))
+
+            suffix = file_name.suffix.lower()
+            if suffix == ".lab":
+                section_name, ok = QtWidgets.QInputDialog.getText(
+                    self,
+                    "Provide .lab file section name",
+                    "Section name:",
+                )
+                if not ok:
+                    section_name = "Selected channels"
+
+            with open(file_name, "w") as output:
+                if suffix == ".lab":
+                    output.write(f"[{section_name}]\n")
+                output.write("\n".join(natsorted(signals)))
+
+    def load_filter_list(self, event=None, file_name=None):
+        if file_name is None:
+            file_name, _ = QtWidgets.QFileDialog.getOpenFileName(
+                self,
+                "Select channel list file",
+                "",
+                "Config file (*.cfg);;TXT files (*.txt);;Display files (*.dsp);;CANape Lab file (*.lab);;All file types (*.cfg *.dsp *.lab *.txt)",
+                "CANape Lab file (*.lab)",
+            )
+
+        if file_name:
+            if not isinstance(file_name, dict):
+                file_name = Path(file_name)
+
+                extension = file_name.suffix.lower()
+                if extension == ".dsp":
+                    info = load_dsp(file_name)
+                    channels = info.get("display", [])
+
+                elif extension == ".lab":
+                    info = load_lab(file_name)
+                    if info:
+                        if len(info) > 1:
+                            section, ok = QtWidgets.QInputDialog.getItem(
+                                None,
+                                "Please select the section",
+                                "Available sections:",
+                                list(info),
+                                0,
+                                False,
+                            )
+                            if ok:
+                                channels = info[section]
+                            else:
+                                return
+                        else:
+                            channels = list(info.values())[0]
+
+                elif extension == ".cfg":
+                    with open(file_name, "r") as infile:
+                        info = json.load(infile)
+                    channels = info.get("selected_channels", [])
+                elif extension == ".txt":
+                    try:
+                        with open(file_name, "r") as infile:
+                            info = json.load(infile)
+                        channels = info.get("selected_channels", [])
+                    except:
+                        with open(file_name, "r") as infile:
+                            channels = [line.strip() for line in infile.readlines()]
+                            channels = [name for name in channels if name]
+
+            else:
+                info = file_name
+                channels = info.get("selected_channels", [])
+
+            if channels:
+
+                iterator = QtWidgets.QTreeWidgetItemIterator(self.filter_tree)
+
+                if self.filter_view.currentText() == "Internal file structure":
+                    while iterator.value():
+                        item = iterator.value()
+                        if item.parent() is None:
+                            iterator += 1
+                            continue
+
+                        channel_name = item.text(0)
+                        if channel_name in channels:
+                            item.setCheckState(0, QtCore.Qt.Checked)
+                            channels.pop(channels.index(channel_name))
+                        else:
+                            item.setCheckState(0, QtCore.Qt.Unchecked)
+
+                        iterator += 1
+                elif self.filter_view.currentText() == "Natural sort":
+                    while iterator.value():
+                        item = iterator.value()
+
+                        channel_name = item.text(0)
+                        if channel_name in channels:
+                            item.setCheckState(0, QtCore.Qt.Checked)
+                            channels.pop(channels.index(channel_name))
+                        else:
+                            item.setCheckState(0, QtCore.Qt.Unchecked)
+
+                        iterator += 1
+
+                else:
+                    items = []
+                    self.filter_tree.clear()
+
+                    for i, gp in enumerate(self.mdf.groups):
+                        for j, ch in enumerate(gp.channels):
+                            if ch.name in channels:
+                                entry = i, j
+                                channel = TreeItem(
+                                    entry, ch.name, origin_uuid=self.uuid
+                                )
+                                channel.setText(0, ch.name)
+                                channel.setCheckState(0, QtCore.Qt.Checked)
+                                items.append(channel)
+
+                                channels.pop(channels.index(ch.name))
+
+                    if len(items) < 30000:
+                        items = natsorted(items, key=lambda x: x.name)
+                    else:
+                        items.sort(key=lambda x: x.name)
+                    self.filter_tree.addTopLevelItems(items)
