@@ -2,10 +2,10 @@
 
 import numpy as np
 import pandas as pd
-from PyQt5 import QtCore, QtGui, QtWidgets
+from PySide6 import QtCore, QtGui, QtWidgets
 
 from ...blocks.utils import csv_bytearray2hex
-from ..ui import resource_rc as resource_rc
+from ..ui import resource_rc
 from ..ui.channel_group_info_widget import Ui_ChannelGroupInfo
 from ..widgets.list_item import ListItem
 
