@@ -3,24 +3,27 @@
 import json
 from struct import pack
 
-from PyQt5 import QtCore, QtGui, QtWidgets
+from natsort import natsorted
+from PySide6 import QtCore, QtGui, QtWidgets
 
 from ..utils import extract_mime_names
 
 
 class ListWidget(QtWidgets.QListWidget):
 
-    itemsDeleted = QtCore.pyqtSignal(list)
-    set_time_offset = QtCore.pyqtSignal(list)
-    items_rearranged = QtCore.pyqtSignal()
-    add_channels_request = QtCore.pyqtSignal(list)
-    show_properties = QtCore.pyqtSignal(object)
-    insert_computation = QtCore.pyqtSignal(str)
+    itemsDeleted = QtCore.Signal(list)
+    set_time_offset = QtCore.Signal(list)
+    items_rearranged = QtCore.Signal()
+    add_channels_request = QtCore.Signal(list)
+    show_properties = QtCore.Signal(object)
+    insert_computation = QtCore.Signal(str)
 
     def __init__(self, *args, **kwargs):
 
         super().__init__(*args, **kwargs)
 
+        self.details_enabled = False
+
         self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
         self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
         self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
@@ -30,7 +33,24 @@ class ListWidget(QtWidgets.QListWidget):
 
         self.can_delete_items = True
         self.setAcceptDrops(True)
+
+        self.itemSelectionChanged.connect(self.item_selection_changed)
+
         self.show()
+        self._has_hidden_items = False
+
+    def item_selection_changed(self, item=None):
+        selection = list(self.selectedItems())
+        for row in range(self.count()):
+            item = self.item(row)
+            if item in selection:
+                widget = self.itemWidget(item)
+                if widget is not None:
+                    widget.set_selected(True)
+            else:
+                widget = self.itemWidget(item)
+                if widget is not None:
+                    widget.set_selected(False)
 
     def keyPressEvent(self, event):
         key = event.key()
@@ -43,6 +63,8 @@ class ListWidget(QtWidgets.QListWidget):
                 row = self.row(item)
                 item_widget = self.itemWidget(item)
                 deleted.append(getattr(item_widget, "uuid", None))
+                if hasattr(item_widget, "disconnect_slots"):
+                    item_widget.disconnect_slots()
                 self.takeItem(row)
             if deleted:
                 self.itemsDeleted.emit(deleted)
@@ -132,7 +154,7 @@ class ListWidget(QtWidgets.QListWidget):
             data.append(
                 pack(
                     f"<12s3q{len(info)}s",
-                    str(item.mdf_uuid).encode("ascii"),
+                    str(item.origin_uuid).encode("ascii"),
                     entry[0],
                     entry[1],
                     len(info),
@@ -171,31 +193,43 @@ class ListWidget(QtWidgets.QListWidget):
         if item is None:
             return
 
-        menu = QtWidgets.QMenu()
-        menu.addAction(self.tr("Copy name (Ctrl+C)"))
-        menu.addAction(self.tr("Copy display properties (Ctrl+Shift+C)"))
-        menu.addAction(self.tr("Paste display properties (Ctrl+Shift+P)"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Enable all"))
-        menu.addAction(self.tr("Disable all"))
-        menu.addAction(self.tr("Enable all but this"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Add to common Y axis"))
-        menu.addAction(self.tr("Remove from common Y axis"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Set unit"))
-        menu.addAction(self.tr("Set precision"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Relative time base shift"))
-        menu.addAction(self.tr("Set time base start offset"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Insert computation using this channel"))
-        menu.addSeparator()
-        menu.addAction(self.tr("Delete (Del)"))
-        menu.addSeparator()
-        menu.addAction(self.tr("File/Computation properties"))
+        else:
 
-        action = menu.exec_(self.viewport().mapToGlobal(position))
+            menu = QtWidgets.QMenu()
+            menu.addAction(self.tr(f"{self.count()} items in the list"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Copy name (Ctrl+C)"))
+            menu.addAction(self.tr("Copy display properties (Ctrl+Shift+C)"))
+            menu.addAction(self.tr("Paste display properties (Ctrl+Shift+P)"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Enable all"))
+            menu.addAction(self.tr("Disable all"))
+            menu.addAction(self.tr("Enable all but this"))
+            menu.addSeparator()
+            if self._has_hidden_items:
+                show_hide = "Show disabled items"
+            else:
+                show_hide = "Hide disabled items"
+            menu.addAction(self.tr(show_hide))
+            menu.addSeparator()
+
+            menu.addAction(self.tr("Add to common Y axis"))
+            menu.addAction(self.tr("Remove from common Y axis"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Set unit"))
+            menu.addAction(self.tr("Set precision"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Relative time base shift"))
+            menu.addAction(self.tr("Set time base start offset"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Insert computation using this channel"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Delete (Del)"))
+            menu.addSeparator()
+            menu.addAction(self.tr("Toggle details"))
+            menu.addAction(self.tr("File/Computation properties"))
+
+            action = menu.exec_(self.viewport().mapToGlobal(position))
 
         if action is None:
             return
@@ -244,6 +278,20 @@ class ListWidget(QtWidgets.QListWidget):
                 else:
                     widget.display.setCheckState(QtCore.Qt.Checked)
 
+        elif action.text() == show_hide:
+            if self._has_hidden_items:
+                for i in range(self.count()):
+                    item = self.item(i)
+                    item.setHidden(False)
+            else:
+                for i in range(self.count()):
+                    item = self.item(i)
+                    widget = self.itemWidget(item)
+                    if not widget.display.isChecked():
+                        item.setHidden(True)
+
+            self._has_hidden_items = not self._has_hidden_items
+
         elif action.text() == "Add to common Y axis":
             selected_items = self.selectedItems()
             for i in range(self.count()):
@@ -300,12 +348,15 @@ class ListWidget(QtWidgets.QListWidget):
 
                 if action.text() == "Relative time base shift":
                     offset, ok = QtWidgets.QInputDialog.getDouble(
-                        self, "Relative offset [s]", "Offset [s]:"
+                        self, "Relative offset [s]", "Offset [s]:", decimals=6
                     )
                     absolute = False
                 else:
                     offset, ok = QtWidgets.QInputDialog.getDouble(
-                        self, "Absolute time start offset [s]", "Offset [s]:"
+                        self,
+                        "Absolute time start offset [s]",
+                        "Offset [s]:",
+                        decimals=6,
                     )
                     absolute = True
                 if ok:
@@ -325,6 +376,14 @@ class ListWidget(QtWidgets.QListWidget):
             )
             self.keyPressEvent(event)
 
+        elif action.text() == "Toggle details":
+            self.details_enabled = not self.details_enabled
+            for i in range(self.count()):
+                item = self.item(i)
+                widget = self.itemWidget(item)
+                widget.details.setVisible(self.details_enabled)
+                item.setSizeHint(widget.sizeHint())
+
         elif action.text() == "File/Computation properties":
             selected_items = self.selectedItems()
             if len(selected_items) == 1:
@@ -340,14 +399,15 @@ class ListWidget(QtWidgets.QListWidget):
 
 class MinimalListWidget(QtWidgets.QListWidget):
 
-    itemsDeleted = QtCore.pyqtSignal(list)
-    itemsPasted = QtCore.pyqtSignal()
+    itemsDeleted = QtCore.Signal(list)
+    itemsDeleted = QtCore.Signal(list)
+    itemsPasted = QtCore.Signal()
 
     def __init__(self, *args, **kwargs):
 
         super().__init__(*args, **kwargs)
 
-        self.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
+        self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
         self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
 
         self.setAlternatingRowColors(True)
@@ -358,20 +418,65 @@ class MinimalListWidget(QtWidgets.QListWidget):
         self.setAcceptDrops(True)
         self.show()
 
+        self.itemSelectionChanged.connect(self.item_selection_changed)
+
         self.minimal_menu = False
+        self.all_texts = False
+        self.placeholder_text = ""
+
+        self.user_editable = True
+
+    def item_selection_changed(self, item=None):
+        try:
+            selection = list(self.selectedItems())
+            for row in range(self.count()):
+                item = self.item(row)
+                if item in selection:
+                    self.itemWidget(item).set_selected(True)
+                else:
+                    self.itemWidget(item).set_selected(False)
+        except:
+            pass
 
     def keyPressEvent(self, event):
         key = event.key()
         modifiers = event.modifiers()
-        if key == QtCore.Qt.Key_Delete:
+        if key == QtCore.Qt.Key_Delete and self.user_editable:
             selected_items = self.selectedItems()
             deleted = []
-            for item in selected_items:
-                row = self.row(item)
-                deleted.append(row)
-                self.takeItem(row)
+
+            if self.all_texts:
+                to_delete = set()
+                for item in selected_items:
+                    row = self.row(item)
+                    deleted.append(row)
+                    item_widget = self.itemWidget(item)
+                    if hasattr(item_widget, "disconnect_slots"):
+                        item_widget.disconnect_slots()
+                    to_delete.add(item.text())
+
+                all_texts = set()
+
+                count = self.count()
+                for row in range(count):
+                    item = self.item(row)
+                    all_texts.add(item.text())
+
+                self.clear()
+                self.addItems(natsorted(all_texts - to_delete))
+
+            else:
+                for item in selected_items:
+                    row = self.row(item)
+                    deleted.append(row)
+                    item_widget = self.itemWidget(item)
+                    if hasattr(item_widget, "disconnect_slots"):
+                        item_widget.disconnect_slots()
+                    self.takeItem(row)
+
             if deleted:
                 self.itemsDeleted.emit(deleted)
+
         elif key == QtCore.Qt.Key_C and modifiers == QtCore.Qt.ControlModifier:
             try:
                 text = [item.text() for item in self.selectedItems()]
@@ -379,7 +484,11 @@ class MinimalListWidget(QtWidgets.QListWidget):
             except:
                 text = ""
             QtWidgets.QApplication.instance().clipboard().setText(text)
-        elif key == QtCore.Qt.Key_V and modifiers == QtCore.Qt.ControlModifier:
+        elif (
+            key == QtCore.Qt.Key_V
+            and modifiers == QtCore.Qt.ControlModifier
+            and self.user_editable
+        ):
             lines = QtWidgets.QApplication.instance().clipboard().text().splitlines()
             if lines:
                 try:
@@ -395,18 +504,25 @@ class MinimalListWidget(QtWidgets.QListWidget):
 
         if self.minimal_menu:
             if self.count() > 0:
+                menu.addAction(self.tr(f"{self.count()} items in the list"))
+                menu.addSeparator()
                 menu.addAction(self.tr("Delete (Del)"))
             else:
                 return
         else:
             if self.count() == 0:
-                menu.addAction(self.tr("Paste names (Ctrl+V)"))
-            else:
-                menu = QtWidgets.QMenu()
-                menu.addAction(self.tr("Copy names (Ctrl+C)"))
-                menu.addAction(self.tr("Paste names (Ctrl+V)"))
+                menu.addAction(self.tr(f"{self.count()} items in the list"))
                 menu.addSeparator()
-                menu.addAction(self.tr("Delete (Del)"))
+                if self.user_editable:
+                    menu.addAction(self.tr("Paste names (Ctrl+V)"))
+            else:
+                menu.addAction(self.tr(f"{self.count()} items in the list"))
+                menu.addSeparator()
+                menu.addAction(self.tr("Copy names (Ctrl+C)"))
+                if self.user_editable:
+                    menu.addAction(self.tr("Paste names (Ctrl+V)"))
+                    menu.addSeparator()
+                    menu.addAction(self.tr("Delete (Del)"))
 
         action = menu.exec_(self.viewport().mapToGlobal(position))
 
@@ -428,3 +544,17 @@ class MinimalListWidget(QtWidgets.QListWidget):
                 QtCore.QEvent.KeyPress, QtCore.Qt.Key_V, QtCore.Qt.ControlModifier
             )
             self.keyPressEvent(event)
+
+    def paintEvent(self, event):
+        super().paintEvent(event)
+        if self.count() == 0 and self.placeholder_text:
+            painter = QtGui.QPainter(self.viewport())
+            painter.save()
+            col = self.palette().placeholderText().color()
+            painter.setPen(col)
+            fm = self.fontMetrics()
+            elided_text = fm.elidedText(
+                self.placeholder_text, QtCore.Qt.ElideRight, self.viewport().width()
+            )
+            painter.drawText(self.viewport().rect(), QtCore.Qt.AlignCenter, elided_text)
+            painter.restore()
