@@ -2,15 +2,16 @@
 import json
 from struct import pack
 
-from PyQt5 import QtCore, QtGui, QtWidgets
+from PySide6 import QtCore, QtGui, QtWidgets
 
+from ..dialogs.range_editor import RangeEditor
 from ..utils import extract_mime_names
 
 
 class NumericTreeWidget(QtWidgets.QTreeWidget):
-    add_channels_request = QtCore.pyqtSignal(list)
-    items_rearranged = QtCore.pyqtSignal()
-    items_deleted = QtCore.pyqtSignal(list)
+    add_channels_request = QtCore.Signal(list)
+    items_rearranged = QtCore.Signal()
+    items_deleted = QtCore.Signal(list)
 
     def __init__(self, *args, **kwargs):
 
@@ -21,34 +22,23 @@ class NumericTreeWidget(QtWidgets.QTreeWidget):
         self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
         self.setEditTriggers(QtWidgets.QAbstractItemView.DoubleClicked)
 
+        self.header().sortIndicatorChanged.connect(self.handle_sorting_changed)
+        self.itemDoubleClicked.connect(self.handle_item_double_click)
+
+        self._handles_double_click = True
+        self.set_double_clicked_enabled(True)
+
+    def set_double_clicked_enabled(self, state):
+        self._handles_double_click = bool(state)
+
     def keyPressEvent(self, event):
         key = event.key()
-        if key == QtCore.Qt.Key_Space:
-            selected_items = self.selectedItems()
-            if not selected_items:
-                return
-            elif len(selected_items) == 1:
-                item = selected_items[0]
-                checked = item.checkState(0)
-                if checked == QtCore.Qt.Checked:
-                    item.setCheckState(0, QtCore.Qt.Unchecked)
-                else:
-                    item.setCheckState(0, QtCore.Qt.Checked)
-            else:
-                if any(
-                    item.checkState(0) == QtCore.Qt.Unchecked for item in selected_items
-                ):
-                    checked = QtCore.Qt.Checked
-                else:
-                    checked = QtCore.Qt.Unchecked
-                for item in selected_items:
-                    item.setCheckState(0, checked)
-        elif (
+        if (
             event.key() == QtCore.Qt.Key_Delete
             and event.modifiers() == QtCore.Qt.NoModifier
         ):
             selected = reversed(self.selectedItems())
-            names = [(item.mdf_uuid, item.text(0)) for item in selected]
+            names = [(item.origin_uuid, item.text(0)) for item in selected]
             for item in selected:
                 if item.parent() is None:
                     index = self.indexFromItem(item).row()
@@ -73,26 +63,29 @@ class NumericTreeWidget(QtWidgets.QTreeWidget):
             if entry == (-1, -1):
                 info = {
                     "name": item.name,
-                    "computation": {},
+                    "computation": item.computation,
                 }
-                info = json.dumps(info).encode("utf-8")
             else:
-                info = item.name.encode("utf-8")
+                info = item.name
+
+            ranges = [dict(e) for e in item.ranges]
+
+            for range_info in ranges:
+                range_info["color"] = range_info["color"].color().name()
 
             data.append(
-                pack(
-                    f"<12s3q{len(info)}s",
-                    str(item.mdf_uuid).encode("ascii"),
-                    entry[0],
-                    entry[1],
-                    len(info),
+                (
                     info,
+                    *item.entry,
+                    str(item.origin_uuid),
+                    "channel",
+                    ranges,
                 )
             )
 
-        mimeData.setData(
-            "application/octet-stream-asammdf", QtCore.QByteArray(b"".join(data))
-        )
+        data = json.dumps(data).encode("utf-8")
+
+        mimeData.setData("application/octet-stream-asammdf", QtCore.QByteArray(data))
 
         drag = QtGui.QDrag(self)
         drag.setMimeData(mimeData)
@@ -113,3 +106,24 @@ class NumericTreeWidget(QtWidgets.QTreeWidget):
                 self.add_channels_request.emit(names)
             else:
                 super().dropEvent(e)
+
+    def handle_item_double_click(self, item, column):
+        if self._handles_double_click:
+            dlg = RangeEditor(
+                item.name,
+                ranges=item.ranges,
+                parent=self,
+                brush=True,
+            )
+            dlg.exec_()
+            if dlg.pressed_button == "apply":
+                item.ranges = dlg.result
+                item.check_signal_range()
+
+    def handle_sorting_changed(self, index, order):
+        iterator = QtWidgets.QTreeWidgetItemIterator(self)
+        while iterator.value():
+            item = iterator.value()
+            iterator += 1
+
+            item._sorting_column = index
