@@ -59,15 +59,17 @@ class TestCANBusLogging(unittest.TestCase):
             if input_file.suffix == ".npy"
         ]
 
-        out = mdf.extract_bus_logging({"CAN": [dbc]})
+        out = mdf.extract_bus_logging({"CAN": [(dbc, 0)]})
 
         for signal in signals:
             name = signal.stem
 
             target = np.load(signal)
-            values = out.get(name).samples
+            sig = out.get(name)
 
-            self.assertTrue(np.array_equal(values, target))
+            self.assertTrue(
+                np.array_equal(sig.samples, target), f"{name} {sig} {target}"
+            )
 
     def test_j1939_extract(self):
         print("J1939 extract")
@@ -94,7 +96,7 @@ class TestCANBusLogging(unittest.TestCase):
             if input_file.suffix == ".npy"
         ]
 
-        out = mdf.extract_bus_logging({"CAN": [dbc]})
+        out = mdf.extract_bus_logging({"CAN": [(dbc, 0)]})
 
         for signal in signals:
             name = signal.stem
