@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 from pathlib import Path
+import shutil
 import tempfile
 import unittest
 
@@ -38,8 +39,8 @@ class TestEndianess(unittest.TestCase):
             mdf.close()
 
             with MDF(outfile) as mdf:
-                self.assertTrue(np.array_equal(mdf.get("Motorola").samples, [2] * 15))
-                self.assertTrue(np.array_equal(mdf.get("Intel").samples, [4] * 15))
+                assert np.array_equal(mdf.get("Motorola").samples, [2] * 15)
+                assert np.array_equal(mdf.get("Intel").samples, [4] * 15)
 
         for version in ("3.30", "4.10"):
             mdf = MDF(version=version)
@@ -50,8 +51,8 @@ class TestEndianess(unittest.TestCase):
             mdf.close()
 
             with MDF(outfile) as mdf:
-                self.assertTrue(np.array_equal(mdf.get("Motorola").samples, [2] * 15))
-                self.assertTrue(np.array_equal(mdf.get("Intel").samples, [4] * 15))
+                assert np.array_equal(mdf.get("Motorola").samples, [2] * 15)
+                assert np.array_equal(mdf.get("Intel").samples, [4] * 15)
 
     def test_not_aligned_mdf_v3(self):
 
@@ -93,13 +94,10 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x0204] * 15)
-                )
-
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples, [0x0204] * 15
                 )
+                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)
 
             ch3 = mdf_source.groups[0].channels[3]
             ch3.start_offset = 16 + t.itemsize * 8
@@ -116,13 +114,12 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15
-                    )
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15
                 )
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedIntel").samples, [0x04023F] * 15)
+
+                assert np.array_equal(
+                    mdf.get("NotAlignedIntel").samples, [0x04023F] * 15
                 )
 
             ch3 = mdf_source.groups[0].channels[3]
@@ -140,17 +137,14 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedMotorola").samples,
-                        [(0x3F0204F8 >> 5) & (2 ** 21 - 1)] * 15,
-                    )
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples,
+                    [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                 )
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedIntel").samples,
-                        [(0xF8040200 >> 6) & (2 ** 21 - 1)] * 15,
-                    )
+
+                assert np.array_equal(
+                    mdf.get("NotAlignedIntel").samples,
+                    [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                 )
 
     def test_not_aligned_mdf_v4(self):
@@ -193,13 +187,11 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedMotorola").samples, [0x0204] * 15)
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples, [0x0204] * 15
                 )
 
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)
-                )
+                assert np.array_equal(mdf.get("NotAlignedIntel").samples, [0x0402] * 15)
 
             ch3 = mdf_source.groups[0].channels[3]
             ch3.byte_offset = 2 + t.itemsize
@@ -216,13 +208,12 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15
-                    )
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples, [0x3F0204] * 15
                 )
-                self.assertTrue(
-                    np.array_equal(mdf.get("NotAlignedIntel").samples, [0x04023F] * 15)
+
+                assert np.array_equal(
+                    mdf.get("NotAlignedIntel").samples, [0x04023F] * 15
                 )
 
             ch3 = mdf_source.groups[0].channels[3]
@@ -242,17 +233,14 @@ class TestEndianess(unittest.TestCase):
             )
 
             with MDF(outfile) as mdf:
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedMotorola").samples,
-                        [(0x3F0204F8 >> 5) & (2 ** 21 - 1)] * 15,
-                    )
+                assert np.array_equal(
+                    mdf.get("NotAlignedMotorola").samples,
+                    [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                 )
-                self.assertTrue(
-                    np.array_equal(
-                        mdf.get("NotAlignedIntel").samples,
-                        [(0xF8040200 >> 6) & (2 ** 21 - 1)] * 15,
-                    )
+
+                assert np.array_equal(
+                    mdf.get("NotAlignedIntel").samples,
+                    [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                 )
 
     def test_overlapping_channels_mdf_v3(self):
@@ -306,16 +294,12 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples, [0x0204] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples, [0x0204] * 15
                     )
 
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples, [0x0402] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples, [0x0402] * 15
                     )
 
                 ch3 = mdf_source.groups[0].channels[2]
@@ -333,15 +317,12 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15
                     )
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples, [0x04023F] * 15
-                        )
+
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples, [0x04023F] * 15
                     )
 
                 ch3 = mdf_source.groups[0].channels[2]
@@ -359,17 +340,14 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples,
-                            [(0x3F0204F8 >> 5) & (2 ** 21 - 1)] * 15,
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples,
+                        [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                     )
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples,
-                            [(0xF8040200 >> 6) & (2 ** 21 - 1)] * 15,
-                        )
+
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples,
+                        [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                     )
 
     def test_overlapping_channels_mdf_v4(self):
@@ -426,16 +404,12 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples, [0x0204] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples, [0x0204] * 15
                     )
 
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples, [0x0402] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples, [0x0402] * 15
                     )
 
                 ch3 = mdf_source.groups[0].channels[2]
@@ -455,15 +429,12 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples, [0x3F0204] * 15
                     )
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples, [0x04023F] * 15
-                        )
+
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples, [0x04023F] * 15
                     )
 
                 ch3 = mdf_source.groups[0].channels[2]
@@ -483,17 +454,14 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples,
-                            [(0x3F0204F8 >> 5) & (2 ** 21 - 1)] * 15,
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples,
+                        [(0x3F0204F8 >> 5) & (2**21 - 1)] * 15,
                     )
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingIntel").samples,
-                            [(0xF8040200 >> 6) & (2 ** 21 - 1)] * 15,
-                        )
+
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples,
+                        [(0xF8040200 >> 6) & (2**21 - 1)] * 15,
                     )
 
                 ch3 = mdf_source.groups[0].channels[2]
@@ -513,13 +481,12 @@ class TestEndianess(unittest.TestCase):
                 )
 
                 with MDF(outfile) as mdf:
-                    self.assertTrue(
-                        np.array_equal(
-                            mdf.get("OverlappingMotorola").samples, [0x1] * 15
-                        )
+                    assert np.array_equal(
+                        mdf.get("OverlappingMotorola").samples, [0x1] * 15
                     )
-                    self.assertTrue(
-                        np.array_equal(mdf.get("OverlappingIntel").samples, [0x1] * 15)
+
+                    assert np.array_equal(
+                        mdf.get("OverlappingIntel").samples, [0x1] * 15
                     )
 
 
