@@ -1,11 +1,13 @@
 #!/usr/bin/env python
-import unittest
-import tempfile
 from pathlib import Path
+import tempfile
+import unittest
 
 import numpy as np
 
-from asammdf import MDF, MDF2, MDF3, Signal
+from asammdf import MDF, Signal
+from asammdf.blocks.mdf_v2 import MDF2
+from asammdf.blocks.mdf_v3 import MDF3
 
 CHANNEL_LEN = 10000
 
@@ -24,13 +26,13 @@ class TestMDF23(unittest.TestCase):
 
     def test_read_mdf2_00(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 2.00 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 15), -1, CHANNEL_LEN),
+            np.random.randint(-(2**15), -1, CHANNEL_LEN),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
@@ -56,13 +58,13 @@ class TestMDF23(unittest.TestCase):
 
     def test_read_mdf2_14(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 2.14 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 29), 2 ** 29, CHANNEL_LEN),
+            np.random.randint(-(2**29), 2**29, CHANNEL_LEN),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
@@ -88,13 +90,13 @@ class TestMDF23(unittest.TestCase):
 
     def test_read_mdf3_00(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 3.00 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 16), 2 ** 16, CHANNEL_LEN, np.int32),
+            np.random.randint(-(2**16), 2**16, CHANNEL_LEN, np.int32),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
@@ -120,13 +122,13 @@ class TestMDF23(unittest.TestCase):
 
     def test_read_mdf3_10(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 3.10 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 9), 2 ** 7, CHANNEL_LEN),
+            np.random.randint(-(2**9), 2**7, CHANNEL_LEN),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
