@@ -5,7 +5,8 @@ import unittest
 
 import numpy as np
 
-from asammdf import MDF, MDF4, Signal
+from asammdf import MDF, Signal
+from asammdf.blocks.mdf_v4 import MDF4
 
 CHANNEL_LEN = 100000
 
@@ -23,13 +24,13 @@ class TestMDF4(unittest.TestCase):
 
     def test_read_mdf4_00(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 4.00 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 31), 2 ** 31, CHANNEL_LEN),
+            np.random.randint(-(2**31), 2**31, CHANNEL_LEN),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
@@ -55,13 +56,13 @@ class TestMDF4(unittest.TestCase):
 
     def test_read_mdf4_10(self):
 
-        seed = np.random.randint(0, 2 ** 31)
+        seed = np.random.randint(0, 2**31)
 
         np.random.seed(seed)
         print("Read 4.10 using seed =", seed)
 
         sig_int = Signal(
-            np.random.randint(-(2 ** 31), 2 ** 31, CHANNEL_LEN),
+            np.random.randint(-(2**31), 2**31, CHANNEL_LEN),
             np.arange(CHANNEL_LEN),
             name="Integer Channel",
             unit="unit1",
@@ -97,7 +98,7 @@ class TestMDF4(unittest.TestCase):
             embedded=True,
         )
         outfile = mdf.save(
-            Path(TestMDF4.tempdir.name) / "attachment.mf4", overwrite=True,
+            Path(TestMDF4.tempdir.name) / "attachment.mf4", overwrite=True
         )
 
         with MDF(outfile) as attachment_mdf:
@@ -121,7 +122,7 @@ class TestMDF4(unittest.TestCase):
             embedded=True,
         )
         outfile = mdf.save(
-            Path(TestMDF4.tempdir.name) / "attachment.mf4", overwrite=True,
+            Path(TestMDF4.tempdir.name) / "attachment.mf4", overwrite=True
         )
 
         with MDF(outfile) as attachment_mdf:
